﻿Option Strict On

Imports System.ComponentModel
Imports System.Runtime.CompilerServices

Public Class Calendar
    Implements INotifyPropertyChanged

    Public Event DayCLicked(currentDay As Date)
    Public Event AppointmentClicked(appointment As AppointmentVm)
    Private Sub Calendar_Loaded(sender As Object, e As RoutedEventArgs) Handles Me.Loaded



    End Sub

    Private _startDate As Date
    Public Property StartDate() As Date
        Get
            Return _startDate
        End Get
        Set(ByVal value As Date)
            _startDate = value
            RaisePropertyChanged()
            MakeCalendar()
        End Set
    End Property


    Private _endDate As Date
    Public Property EndDate() As Date
        Get
            Return _endDate
        End Get
        Set(ByVal value As Date)
            _endDate = value
            RaisePropertyChanged()
            MakeCalendar()
        End Set
    End Property



    Public Sub MakeCalendar()
        calGrid.ColumnDefinitions.Clear()
        calGrid.RowDefinitions.Clear()

        Dim CalendarVM As CalendarViewModel = CType(Me.DataContext, CalendarViewModel)

        'Wieviele Tage werden es?
        Dim daysCount As Integer = CInt(DateDiff(DateInterval.Day, StartDate, EndDate))
        'Wieviele Personen
        Dim personCount As Integer = CalendarVM.Day.Select(Function(s) s.Person).Distinct.Count

        'Jetzt die Border erstellen
        For o As Integer = 0 To daysCount
            For o1 As Integer = 0 To personCount
                Dim border As New Border With {.BorderBrush = New SolidColorBrush(Colors.Black), .BorderThickness = New Thickness(1, 1, 1, 1)}
                Grid.SetColumn(border, o1)
                Grid.SetRow(border, o)
                calGrid.Children.Add(border)
            Next
        Next


        'Tage und Personen eins Grid einfügen
        For i As Integer = 0 To daysCount
            Dim currDate As Date = DateAdd(DateInterval.Day, i, StartDate)
            calGrid.RowDefinitions.Add(New RowDefinition() With {.Height = New GridLength(0, GridUnitType.Auto), .MinHeight = 30})

            Dim border As New Border With {.Background = New SolidColorBrush(Colors.LightCoral), .BorderBrush = New SolidColorBrush(Colors.Black),
                .BorderThickness = New Thickness(1, 1, 1, 1), .Tag = currDate}
            Dim tb As New TextBlock With {.Text = currDate.ToLongDateString}
            border.Child = tb
            AddHandler border.MouseDown, AddressOf DateBorder_Clicked
            Grid.SetColumn(border, 0)
            Grid.SetRow(border, i + 1)
            calGrid.Children.Add(border)

        Next

        Dim columnIndex As Integer = 1
        calGrid.ColumnDefinitions.Add(New ColumnDefinition() With {.Width = New GridLength(80, GridUnitType.Auto)})
        For Each p As String In CalendarVM.Day.Select(Function(s) s.Person).Distinct.ToList
            calGrid.ColumnDefinitions.Add(New ColumnDefinition() With {.Width = New GridLength(80, GridUnitType.Pixel)})
            Dim border As New Border With {.Background = New SolidColorBrush(Colors.LightCyan), .BorderBrush = New SolidColorBrush(Colors.Black),
                .BorderThickness = New Thickness(1, 1, 1, 1)}
            Dim tb As New TextBlock With {.Text = p, .FontWeight = FontWeights.Bold}
            border.Child = tb
            Grid.SetColumn(border, columnIndex)
            Grid.SetRow(tb, 0)
            calGrid.Children.Add(border)
            columnIndex += 1
        Next

        For Each d As DayVm In CalendarVM.Day.Where(Function(f) f.TheDate > StartDate AndAlso f.TheDate < EndDate).ToList
            Dim stack As New StackPanel

            For Each ap As AppointmentVm In d.Appointments
                Dim apHolder As New AppointmentHolder With {.DataContext = ap}
                AddHandler apHolder.MouseDown, AddressOf Appointment_Clicked
                stack.Children.Add(apHolder)
            Next
            Dim border As New Border With {.BorderBrush = New SolidColorBrush(Colors.Black), .BorderThickness = New Thickness(1, 1, 1, 1)}
            border.Child = stack
            Grid.SetRow(border, CInt(DateDiff(DateInterval.Day, StartDate, d.TheDate)) + 2)
            Dim colNumber As Integer = CalendarVM.Day.Select(Function(s) s.Person).Distinct.ToList.IndexOf(d.Person)
            Grid.SetColumn(border, colNumber + 1)
            calGrid.Children.Add(border)
        Next

    End Sub

    Private Sub Appointment_Clicked(sender As Object, e As MouseButtonEventArgs)
        RaiseEvent AppointmentClicked(DirectCast(DirectCast(sender, AppointmentHolder).DataContext, AppointmentVm))
    End Sub

    Private Sub DateBorder_Clicked(sender As Object, e As MouseButtonEventArgs)
        RaiseEvent DayCLicked(CDate(DirectCast(sender, Border).Tag))
    End Sub

    Private Sub Calendar_DataContextChanged(sender As Object, e As DependencyPropertyChangedEventArgs) Handles Me.DataContextChanged
        StartDate = DateAdd(DateInterval.Day, -10, Now)
        EndDate = DateAdd(DateInterval.Day, 10, Now)
        If e.NewValue IsNot Nothing Then MakeCalendar()
    End Sub

    Protected Overridable Sub RaisePropertyChanged(<CallerMemberName> Optional ByVal prop As String = "")
        If prop IsNot Nothing AndAlso prop.Length > 0 Then RaiseEvent PropertyChanged(Me, New PropertyChangedEventArgs(prop))
    End Sub


    Public Event PropertyChanged As PropertyChangedEventHandler Implements INotifyPropertyChanged.PropertyChanged
End Class
